// WebSocketServer

var webSocketServer = require('ws').Server
var guidGen = require("./guidGenerator.js");


var wss;
var httpServer;


//Database of socket-GUID associations
var clients = new Array();


// method INIT
var init = function(server) {
	httpServer = server;
}


// method START
var start = function() {
	wss = new webSocketServer({server: httpServer});
	console.log("[Websocket] Server started");
}


// method INIT HANDLERS
var initHandlers = function() {
	
	wss.broadcast = function(data, sender) {
		for (var i in this.clients) {
			if (this.clients[i] != sender) {
				this.clients[i].send(data, function(error) {
					if (error != null) {
						console.log("[WebSocket] Error during broadcast");
					}
				});
			}
		}
	};
	
	wss.on('connection', function(ws) {
		console.log("[WebSocket] " + ws._socket.remoteAddress + ":"+ ws._socket.remotePort + " connected");
	    
		// Send to the new client his GUID
		var guid = guidGen.generate();
		ws.send("yourguid|" +guid, function(error) {
			if (error != null) {
				console.log("[WebSocket] Cannot send new GUID");
			}
		});
		// And all the other clients connected
		for (var i=0, len = clients.length; i < len; i++) {
			ws.send("newguid|" +clients[i][1], function(error) {
				if (error != null) {
					console.log("[WebSocket] Cannot send connected clients' GUID");
				}
			});
		}
		clients.push(new Array(ws, guid));
		console.log("[WebSocket] GUID: "+guid);
		
		
		// Broadcast the new GUID to all the others
		wss.broadcast("newguid|"+guid, ws);
		
		
		// Closing handler
	    ws.on('close', function() {
	    	for (var i=0, len = clients.length; i < len; i++) {
	    		if (clients[i][0] == ws) {
	    			wss.broadcast("quit|"+clients[i][1], ws)
	    			console.log("[WebSocket] GUID "+clients[i][1]+" disconnected");
	    			clients.splice(i, 1);
	    			break;
	    		}
	    	}
	    });
	   
	    ws.on('message', function(message, flag) {
	    	if (flag.binary == undefined && flag.masked == true) {
		    	wss.broadcast(message, ws);
	    	}
	    });
	});
}


//Node.js module's methods export
exports.init = init;
exports.start = start;
exports.initHandlers = initHandlers;